Public Class Node
    ' Basic properties.
    ' The node's position.
    Public X As Integer
    Public Y As Integer

    ' Pens and brushes for drawing the node.
    Private Shared m_PenNormal As Pen = Pens.Black
    Private Shared m_PenHighlighted As Pen = Pens.Red
    Private Shared m_PenSelected As Pen = Pens.Yellow
    Private Shared m_BrushNormal As Brush = Brushes.White
    Private Shared m_BrushHighlighted As Brush = Brushes.Red
    Private Shared m_BrushSelected As Brush = Brushes.Yellow

    ' Node radius for drawing.
    Private Shared m_NodeRadius As Integer = 5

    ' Determines whether the link is drawn normally,
    ' highlighted, or selected.
    Public Enum NodeSelectStatus
        Normal
        Highlighted
        Selected
    End Enum
    Public SelectStatus As NodeSelectStatus

    ' Extra properties used by student algorithms.
    Public PreviousNode As Node
    Public BestDistance As Double

    ' Used to translate the data to and from XML.
    Public XmlId As Integer

    ' Constructor.
    Public Sub New(Optional ByVal new_x As Integer = 0, _
                   Optional ByVal new_y As Integer = 0)
        X = new_x
        Y = new_y
    End Sub

    ' Draw the node on the indicated canvas.
    Public Sub Draw(ByVal canvas As Graphics)
        Dim the_pen As Pen
        Dim the_brush As Brush

        ' See which pen and brush we should use.
        Select Case SelectStatus
            Case NodeSelectStatus.Normal
                the_pen = m_PenNormal
                the_brush = m_BrushNormal
            Case NodeSelectStatus.Highlighted
                the_pen = m_PenHighlighted
                the_brush = m_BrushHighlighted
            Case NodeSelectStatus.Selected
                the_pen = m_PenSelected
                the_brush = m_BrushSelected
        End Select

        ' Draw the node.
        With canvas
            .FillEllipse(the_brush, _
                X - m_NodeRadius, Y - m_NodeRadius, _
                m_NodeRadius * 2, m_NodeRadius * 2)
            .DrawEllipse(the_pen, _
                X - m_NodeRadius, Y - m_NodeRadius, _
                m_NodeRadius * 2, m_NodeRadius * 2)
        End With
    End Sub

    ' Return the distance from the node to the point.
    Public Function DistanceToPoint(ByVal test_x As Integer, ByVal test_y As Integer) As Double
        Dim dx As Integer
        Dim dy As Integer

        dx = X - test_x
        dy = Y - test_y
        Return Math.Sqrt(dx * dx + dy * dy)
    End Function

    ' Return True if this point is under the node.
    Public Function IsAt(ByVal test_x As Integer, _
                         ByVal test_y As Integer) As Boolean
        Return (DistanceToPoint(test_x, test_y) <= m_NodeRadius)
    End Function
End Class
